require "behaviours/wander"
require "behaviours/runaway"
require "behaviours/doaction"
require "behaviours/panic"

local STOP_RUN_DIST = 7
local SEE_PLAYER_DIST = 5

local AVOID_PLAYER_DIST = 3
local AVOID_PLAYER_STOP = 6

local SEE_BAIT_DIST = 17
local MAX_WANDER_DIST = 35

local SEE_DIST = 50
local TOOCLOSE = 6


local ratbrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

local function GoHomeAction(inst)
    if inst.components.homeseeker and 
       inst.components.homeseeker.home and 
       inst.components.homeseeker.home:IsValid() and
	   inst.sg:HasStateTag("trapped") == false then
        return BufferedAction(inst, inst.components.homeseeker.home, ACTIONS.GOHOME)
    end
end

local function EatFoodAction(inst)

    local target = FindEntity(inst, SEE_BAIT_DIST, function(item) return inst.components.eater:CanEat(item) and item.components.bait and not item:HasTag("planted") and not (item.components.inventoryitem and item.components.inventoryitem:IsHeld()) end)
    if target then
        local act = BufferedAction(inst, target, ACTIONS.EAT)
        act.validfn = function() return not (target.components.inventoryitem and target.components.inventoryitem:IsHeld()) end
        return act
    end
end


local function EmptyChest(inst)
		local player = GetPlayer()
		local target = FindEntity(inst, SEE_DIST, function(item) 
			if item.prefab == "treasurechest" and 
				item.components.container and not item.grz == true and
				not item.components.container:IsEmpty() then
					return player and player:GetDistanceSqToInst(item) > TOOCLOSE*TOOCLOSE
				end
			end)
		if target and inst:GetDistanceSqToInst(target) > 1.8 then
			---print("target found?")
			inst.components.knownlocations:RememberLocation("chestposxz", Vector3(target.Transform:GetWorldPosition()) )
			local chestpos = inst.components.knownlocations:GetLocation("chestposxz")
		---	print("chestpos x "..chestpos.x)
			return BufferedAction(inst, nil, ACTIONS.WALKTO, nil, chestpos, nil, 0.2)
		end
end

function ratbrain:OnStart()
    local clock = GetClock()
    local stealnode = PriorityNode(
	{
		DoAction(self.inst, function() return EmptyChest(self.inst) end, "steal", true ),        
	}, 2)
	
	
    local root = PriorityNode(
    {
        WhileNode( function() return self.inst.components.health.takingfiredamage end, "OnFire", Panic(self.inst)),
        RunAway(self.inst, "scarytoprey", AVOID_PLAYER_DIST, AVOID_PLAYER_STOP),
        RunAway(self.inst, "scarytoprey", SEE_PLAYER_DIST, STOP_RUN_DIST, nil, true),
        EventNode(self.inst, "gohome", 
            DoAction(self.inst, GoHomeAction, "go home", true )),
        WhileNode(function() return clock and clock:IsDay() end, "IsNight",
            DoAction(self.inst, GoHomeAction, "go home", true )),
			
        DoAction(self.inst, EatFoodAction),

		MinPeriod(self.inst, 10, 
			stealnode),
			
		
			
			
        Wander(self.inst, function() return self.inst.components.knownlocations:GetLocation("home") end, MAX_WANDER_DIST)
    }, .25)
    self.bt = BT(self.inst, root)
end

return ratbrain
